@@ -28,6 +28,7 @@
 #include "xst-report-hook.h"
 #include "xst-platform.h"
 #include "xst-ui.h"
+#include "xst-su.h"
 
 #include <gnome.h>
 #include <parser.h>
@@ -60,6 +61,8 @@ static gboolean report_finished_cb        (XstTool *tool, XstReportLine *rline);
 
 static void report_dispatch (XstTool *tool);
 
+static gint xst_tool_compare_platforms (XstPlatform *a, XstPlatform *b);
+
 static XstReportHookEntry common_report_hooks[] = {
 	{ "end",              report_finished_cb,         XST_REPORT_HOOK_LOADSAVE, TRUE  },
 	{ "platform_list",    platform_add_supported_cb,  XST_REPORT_HOOK_LOAD,     TRUE  }, 
@@ -93,6 +96,12 @@ platform_list_unselect_row_cb (GtkCList *clist, gint row, gint column, GdkEvent
 	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
 }
 
+static void
+platform_dialog_close_cb (GtkWidget *dialog, XstTool *tool)
+{
+	exit (1);
+}
+
 static void
 platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 {
@@ -117,6 +126,11 @@ platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 	tool->run_again = TRUE;
 
 	report_dispatch (tool);
+
+	gtk_signal_disconnect_by_func (GTK_OBJECT (tool->platform_dialog),
+				       platform_dialog_close_cb,
+				       tool);
+	
 }
 
 static gboolean
@@ -151,6 +165,8 @@ platform_unsupported_cb (XstTool *tool, XstReportLine *rline)
 			    platform_list_unselect_row_cb, tool);
 	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "clicked",
 			    platform_unsupported_clicked_cb, tool);
+	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "close",
+			    platform_dialog_close_cb, tool);
 
 	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
 	gtk_widget_show (tool->platform_dialog);
@@ -194,6 +210,14 @@ report_finished_cb (XstTool *tool, XstReportLine *rline)
 	return TRUE;
 }
 
+/* --- Utility function --- */
+
+static gint
+xst_tool_compare_platforms (XstPlatform *a, XstPlatform *b)
+{
+	return strcmp (a->name, b->name);
+}
+
 /* --- XstTool --- */
 
 void
@@ -202,11 +226,15 @@ xst_tool_add_supported_platform (XstTool *tool, XstPlatform *platform)
 	g_return_if_fail (tool != NULL);
 	g_return_if_fail (platform != NULL);
 
-	/* TODO: Avoid duplicates. Backend doesn't serve duplicates, but
-	 * we should be paranoid here. */
+	/* Avoid duplicates. Backend shouldn't serve duplicates, but
+	 * we're paranoid here. */
+	g_return_if_fail
+		(g_slist_find_custom (tool->supported_platforms_list, platform,
+		    (GCompareFunc) xst_tool_compare_platforms) == NULL);
 
 	tool->supported_platforms_list =
-		g_slist_append (tool->supported_platforms_list, platform);
+		g_slist_insert_sorted (tool->supported_platforms_list, platform,
+		    (GCompareFunc) xst_tool_compare_platforms);
 }
 
 void
@@ -263,6 +291,7 @@ xst_tool_get_access (XstTool *tool)
 	return root_access != ROOT_ACCESS_NONE;
 }
 
+#if 0
 static void
 timeout_cb (gpointer data)
 {
@@ -276,8 +305,6 @@ timeout_cb (gpointer data)
 		gtk_main_quit ();
 }
 
-#if 0
-
 static void
 set_arrow (XstTool *tool, GtkArrowType arrow)
 {
@@ -311,13 +338,13 @@ report_dispatch (XstTool *tool)
 {
 	GSList *list;
 	XstReportLine *rline;
+#if 0
 	GtkAdjustment *vadj;
 	char *report_text[] = {
 		NULL,
 		NULL
 	};
 
-#if 0
 	if (!GTK_IS_SCROLLED_WINDOW (tool->report_scrolled)) {
 		g_warning ("tool->report_scrolled is not a GtkScrolledWindow\n");
 		tool->report_dispatch_pending = FALSE;
@@ -444,7 +471,9 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 static void
 report_progress (XstTool *tool, int fd, const gchar *label)
 {
+#if 0
 	gint cb_id;
+#endif	
 
 	tool->timeout_done = FALSE;
 	tool->report_list_visible = FALSE;
@@ -491,7 +520,7 @@ gboolean
 xst_tool_save (XstTool *tool)
 {
 	FILE *f;
-#warning This is a security problem, fix
+#warning This might be a security problem, fix
 	FILE *debug_file;
 	int fd_xml [2], fd_report [2];
 	int t;
@@ -502,7 +531,7 @@ xst_tool_save (XstTool *tool)
 
 	g_return_val_if_fail (root_access != ROOT_ACCESS_NONE, FALSE);
 
-	xst_dialog_freeze (tool->main_dialog);
+	xst_dialog_freeze_visible (tool->main_dialog);
 
 	gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_XML]);
 
@@ -561,7 +590,7 @@ xst_tool_save (XstTool *tool)
 		g_error ("Unable to run backend: %s", tool->script_path);
 	}
 
-	xst_dialog_thaw (tool->main_dialog);
+	xst_dialog_thaw_visible (tool->main_dialog);
 	return TRUE;  /* FIXME: Determine if it really worked. */
 }
 
@@ -673,15 +702,17 @@ xst_tool_load_try (XstTool *tool)
 }
 
 void
-xst_tool_main (XstTool *tool)
+xst_tool_main (XstTool *tool, gboolean no_main_loop)
 {
-	xst_dialog_freeze (tool->main_dialog);
+	xst_dialog_freeze_visible (tool->main_dialog);
 	gtk_widget_show (GTK_WIDGET (tool->main_dialog));
 
 	xst_tool_load_try (tool);
 
-	xst_dialog_thaw (tool->main_dialog);
-	gtk_main ();
+	xst_dialog_thaw_visible (tool->main_dialog);
+
+	if (!no_main_loop)
+		gtk_main ();
 }
 
 static void
@@ -797,8 +828,8 @@ xst_tool_type_init (XstTool *tool)
 #endif
 
 	xml = xst_tool_load_glade_common (tool, "platform_dialog");
-	tool->platform_dialog = glade_xml_get_widget (xml, "platform_dialog");
-	tool->platform_list = glade_xml_get_widget (xml, "platform_list");
+	tool->platform_dialog    = glade_xml_get_widget (xml, "platform_dialog");
+	tool->platform_list      = glade_xml_get_widget (xml, "platform_list");
 	tool->platform_ok_button = glade_xml_get_widget (xml, "platform_ok_button");
 }
 
@@ -935,6 +966,48 @@ xst_tool_reset_report_hooks (XstTool *tool)
 		((XstReportHook *) list->data)->invoked = FALSE;
 }
 
+/**
+ * xst_fool_the_linker:
+ * @void: 
+ * 
+ * We need to keep the symbol for the create image widget function
+ * so that libglade can find it to create the icons.
+ **/
+void xst_fool_the_linker (void);
+void
+xst_fool_the_linker (void)
+{
+	xst_ui_create_image_widget (NULL, NULL, NULL, 0, 0);
+}
+
+static void
+authenticate (gchar *exec_path)
+{
+	GtkWidget *error_dialog;
+	gchar *password;
+	gint result;
+
+	for (;;)
+	{
+		result = xst_su_get_password (&password);
+
+		if (result < 0)
+			exit (0);
+		else if (result == 0)
+			return;
+
+		/* If successful, the following never returns */
+
+		xst_su_run_with_password (exec_path, password);
+
+		if (strlen (password))
+			memset (password, 0, strlen (password));
+
+		error_dialog = gnome_error_dialog (_("The password you entered is invalid."));
+		gnome_dialog_run_and_close (GNOME_DIALOG (error_dialog));
+	}
+}
+
 void
 xst_init (const gchar *app_name, int argc, char *argv [], const poptOption options)
 {
@@ -945,11 +1018,6 @@ xst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 	textdomain (PACKAGE);
 #endif
 
-	/* This is to fool the linker: do not delete, even if it doesn't
-	 * make sense. Arturo Espinosa <arturo@ximian.com>
-	 */
-	xst_ui_create_image_widget (NULL, NULL, NULL, 0, 0);
-
 	if (options == NULL) {
 		gnome_init (app_name, VERSION, argc, argv);
 	} else {
@@ -979,16 +1047,7 @@ xst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 		root_access = ROOT_ACCESS_SIMULATED;
 #endif
 	} else {
-		d = gnome_ok_cancel_dialog (_("You need full administration privileges (i.e. root)\n"
-					      "to run this configuration tool. You can acquire\n"
-					      "such privileges by running it from the GNOME Control\n"
-					      "Center or issuing an \"su\" command in a shell.\n\n"
-					      "You will be unable to make any changes. Continue anyway?"), 
-					    NULL, NULL);
-
-		if (gnome_dialog_run_and_close (GNOME_DIALOG (d)))
-			exit (1);
-		
+		authenticate (argv [0]);
 		root_access = ROOT_ACCESS_NONE;
 	}
 }
