@@ -26,7 +26,7 @@
 #include <string.h>
 #include <gnome.h>
 
-#include "global.h"
+#include "xst.h"
 
 #include "callbacks.h"
 #include "transfer.h"
@@ -37,11 +37,6 @@ extern XstTool *tool;
 GtkCTreeNode *disks_node_selected = NULL;
 
 
-/* --- Prototypes --- */
-
-
-
-
 /* --- */
 
 
@@ -94,10 +89,12 @@ file_system_id_map[] =
 {
 	{ 0, "auto" },
 	{ 1, "ext2" },
-	{ 2, "fat" },
-	{ 3, "vfat" },
-	{ 4, "fat32" },
-	{ 5, "ntfs" },
+	{ 2, "reiserfs" },
+	{ 3, "ufs" },
+	{ 4, "fat" },
+	{ 5, "vfat" },
+	{ 6, "fat32" },
+	{ 7, "ntfs" },
 	{ 0, NULL }
 };
 
@@ -129,7 +126,7 @@ file_system_description_to_string (char *desc)
   
 	for (i = 0; file_system_id_map [i].string; i++)
 	{
-		if (strcasestr(desc, file_system_id_map [i].string))
+		if (xst_util_strcasestr (desc, file_system_id_map [i].string))
 		{
 			if (strlen(file_system_id_map [i].string) > len)
 			{
@@ -220,7 +217,9 @@ on_partition_type_selected (GtkMenuShell *menu, gpointer data)
 {
 	gchar *s0;
 
-	gtk_label_get (GTK_LABEL (GTK_BIN (xst_dialog_get_widget (tool->main_dialog, "partition_type"))->child), &s0);
+	gtk_label_get (GTK_LABEL
+		       (GTK_BIN
+			(xst_dialog_get_widget (tool->main_dialog, "partition_type"))->child), &s0);
 #if 0
 	if (strstr (s0, "NFS"))
 	{
@@ -255,13 +254,16 @@ partition_dialog_prepare_from_selected ()
 	{
 		/* Get partition type from XML */
 
-		gtk_option_menu_set_history (GTK_OPTION_MENU (xst_dialog_get_widget (tool->main_dialog, "partition_type")), 0);
+		gtk_option_menu_set_history (
+			GTK_OPTION_MENU (xst_dialog_get_widget (tool->main_dialog, "partition_type")), 0);
 
 		if ((n0 = xst_xml_element_find_first (node, "type")) &&
 		    (text = xst_xml_element_get_content (n0)))
 		{
-			gtk_option_menu_set_history (GTK_OPTION_MENU (xst_dialog_get_widget (tool->main_dialog, "partition_type")),
-						     file_system_string_to_id(text));
+			gtk_option_menu_set_history (
+				GTK_OPTION_MENU (
+					xst_dialog_get_widget (tool->main_dialog, "partition_type")),
+				file_system_string_to_id(text));
 		}
 
 		/* Round up the known partitions and add them to the option list of
