@@ -29,7 +29,7 @@
 #include "connection.h"
 #include "callbacks.h"
 
-#include "global.h"
+#include "xst.h"
 
 typedef struct _XstNetworkInterfaceDescription XstNetworkInterfaceDescription;
 
@@ -60,7 +60,7 @@ static void on_connection_config_dialog_destroy (GtkWidget *w, XstConnection *cx
 static gint on_connection_config_dialog_delete_event (GtkWidget *w, GdkEvent *evt, XstConnection *cxn);
 static void on_connection_modified (GtkWidget *w, XstConnection *cxn);
 static void on_wvlan_adhoc_toggled (GtkWidget *w, XstConnection *cxn);
-static void on_ppp_peerdns_toggled (GtkWidget *w, XstConnection *cxn);
+static void on_ppp_update_dns_toggled (GtkWidget *w, XstConnection *cxn);
 
 #define W(s) my_get_widget (cxn->xml, (s))
 
@@ -171,12 +171,15 @@ connection_xml_wvsection_has_name (xmlNode *node, gchar *name)
 {
 	gchar *section_found;
 	gint cmp;
-	
-	section_found = xst_xml_get_child_content (node, "name");
-	cmp = strcmp (section_found, name);
-	g_free (section_found);
 
-	return !cmp;
+	section_found = xst_xml_get_child_content (node, "name");
+	if (section_found) {
+		cmp = strcmp (section_found, name);
+		g_free (section_found);
+		return !cmp;
+	}
+
+	return 0;
 }
 
 /* Use type == NULL if you don't care about the type. */
@@ -479,6 +482,9 @@ connection_init_gui (XstTool *tool)
 void
 connection_set_row_pixtext (GtkWidget *clist, gint row, gchar *text, gboolean enabled)
 {
+	g_return_if_fail (GTK_IS_CLIST (clist));
+	g_return_if_fail (text != NULL);
+	
 	gtk_clist_set_pixtext (GTK_CLIST (clist), row, 1,
 			       text,
 			       GNOME_PAD_SMALL,
@@ -493,17 +499,25 @@ connection_update_row_enabled (XstConnection *cxn, gboolean enabled)
 	gint row, i;
 	XstConnection *cxn2;
 
+	g_return_if_fail (cxn != NULL);
+	g_return_if_fail (cxn->dev != NULL);
+	
 	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
 
 	if (enabled)
 		for (i = 0; i < GTK_CLIST (clist)->rows; i++) {
 			cxn2 = gtk_clist_get_row_data (GTK_CLIST (clist), i);
+
+			g_return_if_fail (cxn2 != NULL);
+			g_return_if_fail (cxn2->dev != NULL);
+			
 			if (!strcmp (cxn->dev, cxn2->dev))
 				connection_update_row_enabled (cxn2, FALSE);
 		}
 
 	cxn->enabled = enabled;
 	row = gtk_clist_find_row_from_data (GTK_CLIST (clist), cxn);
+	g_return_if_fail (row > -1);
 	connection_set_row_pixtext (clist, row, enabled ? _("To activate") :
 				    _("To inactivate"), enabled);
 	xst_dialog_modify (tool->main_dialog);
@@ -515,10 +529,16 @@ connection_update_row (XstConnection *cxn)
 	GtkWidget *clist;
 	gint row;
 
+	g_return_if_fail (cxn != NULL);
+	
 	clist = xst_dialog_get_widget (tool->main_dialog, "connection_list");
 
 	row = gtk_clist_find_row_from_data (GTK_CLIST (clist), cxn);
 
+	g_return_if_fail (row > -1);
+	g_return_if_fail (GTK_IS_CLIST (clist));
+	g_return_if_fail (cxn->dev != NULL);
+
 	gtk_clist_set_pixtext (GTK_CLIST (clist), row, 0, cxn->dev, GNOME_PAD_SMALL,
 			       mini_pm[cxn->type], mini_mask[cxn->type]);
 
@@ -532,7 +552,10 @@ void
 connection_add_to_list (XstConnection *cxn, GtkWidget *clist)
 {
 	int row;
-	char *text[3] = { NULL };
+	char *text[3] = { "Error", NULL };
+
+	g_return_if_fail (GTK_IS_CLIST (clist));
+	g_return_if_fail (cxn != NULL);
 
 	row = gtk_clist_append (GTK_CLIST (clist), text);
 	gtk_clist_set_row_data (GTK_CLIST (clist), row, cxn);
@@ -558,7 +581,7 @@ connection_description_from_type (XstConnectionType type)
 		if (type == xst_iface_desc[i].type)
 			break;
 
-	return g_strdup (xst_iface_desc[i].description);
+	return g_strdup (_(xst_iface_desc[i].description));
 }
 
 extern gchar *
@@ -591,7 +614,6 @@ connection_get_ppp_from_node (xmlNode *node, XstConnection *cxn)
 	cxn->persist = connection_xml_get_boolean (node, "persist");
 	cxn->serial_port = xst_xml_get_child_content (node, "serial_port");
 	cxn->set_default_gw = connection_xml_get_boolean (node, "set_default_gw");
-	cxn->peerdns = connection_xml_get_boolean (node, "peerdns");
 	cxn->dns1 = xst_xml_get_child_content (node, "dns1");
 	cxn->dns2 = xst_xml_get_child_content (node, "dns2");
 	cxn->ppp_options = xst_xml_get_child_content (node, "ppp_options");
@@ -614,7 +636,7 @@ connection_new_from_type (XstConnectionType type, xmlNode *root)
 	/* set up some defaults */
 	cxn->user = FALSE;
 	cxn->autoboot = TRUE;
-	cxn->dhcp_dns = TRUE;
+	cxn->update_dns = TRUE;
 	cxn->ip_config = cxn->tmp_ip_config = IP_MANUAL;
 	
 #warning FIXME: figure out a new device correctly
@@ -627,7 +649,6 @@ connection_new_from_type (XstConnectionType type, xmlNode *root)
 		break;
 	case XST_CONNECTION_PPP:
 		cxn->user = TRUE;
-		cxn->peerdns = TRUE;
 		cxn->autoboot = FALSE;
 		cxn->dev = connection_dev_get_next (root, "ppp");
 		break;
@@ -660,8 +681,9 @@ connection_new_from_node (xmlNode *node)
 		cxn = connection_new_from_dev_name (s, node->parent);
 		g_free (cxn->dev);
 		cxn->dev = s;
-	} else
+	} else {
 		cxn = connection_new_from_type (XST_CONNECTION_OTHER, node->parent);
+	}
 
 	cxn->node = node;
 	
@@ -681,6 +703,7 @@ connection_new_from_node (xmlNode *node)
 	cxn->user = connection_xml_get_boolean (node, "user");
 	cxn->autoboot = connection_xml_get_boolean (node, "auto");
 	cxn->enabled = connection_xml_get_boolean (node, "enabled");
+	cxn->update_dns = connection_xml_get_boolean (node, "update_dns");
 
 	/* TCP/IP general paramaters */
 	cxn->address = xst_xml_get_child_content (node, "address");
@@ -783,7 +806,7 @@ static void
 empty_ip (XstConnection *cxn)
 {
 	cxn->ip_config = cxn->tmp_ip_config;
-	GET_BOOL ("status_", dhcp_dns);
+	GET_BOOL ("ip_", update_dns);
 	GET_STR ("ip_", address);
 	GET_STR ("ip_", netmask);
 	GET_STR ("ip_", gateway);
@@ -811,7 +834,7 @@ empty_ppp_adv (XstConnection *cxn)
 	GET_STR ("ppp_", serial_port);
 	GET_BOOL ("ppp_", stupid);
 	GET_BOOL ("ppp_", set_default_gw);
-	GET_BOOL_NOT ("ppp_", peerdns);
+	GET_BOOL_NOT ("ppp_", update_dns);
 	GET_STR ("ppp_", dns1);
 	GET_STR ("ppp_", dns2);
 	GET_STR ("ppp_", ppp_options);
@@ -870,12 +893,15 @@ static void
 on_connection_cancel_clicked (GtkWidget *w, XstConnection *cxn)
 {
 	GtkCList *list;
+	gint row;
 
 	gtk_widget_destroy (cxn->window);
 	
 	if (cxn->creating) {
 		list = GTK_CLIST (xst_dialog_get_widget (tool->main_dialog, "connection_list"));
-		gtk_clist_remove (list, gtk_clist_find_row_from_data (list, cxn));
+		row = gtk_clist_find_row_from_data (list, cxn);
+		g_return_if_fail (row > -1);
+		gtk_clist_remove (list, row);
 		connection_free (cxn);
 		xst_dialog_modify (tool->main_dialog);
 	}
@@ -909,11 +935,11 @@ on_wvlan_adhoc_toggled (GtkWidget *w, XstConnection *cxn)
 }
 
 static void
-on_ppp_peerdns_toggled (GtkWidget *w, XstConnection *cxn)
+on_ppp_update_dns_toggled (GtkWidget *w, XstConnection *cxn)
 {
 	gboolean active;
 
-	active = GTK_TOGGLE_BUTTON (W ("ppp_peerdns"))->active;
+	active = GTK_TOGGLE_BUTTON (W ("ppp_update_dns"))->active;
 
 	gtk_widget_set_sensitive (W ("ppp_dns1_label"), active);
 	gtk_widget_set_sensitive (W ("ppp_dns2_label"), active);
@@ -939,8 +965,8 @@ update_ip_config (XstConnection *cxn)
 
 	ip = cxn->tmp_ip_config;
 
-	SET_BOOL ("status_", dhcp_dns);
-	gtk_widget_set_sensitive (W ("status_dhcp_dns"), ip != IP_MANUAL);
+	SET_BOOL ("ip_", update_dns);
+	gtk_widget_set_sensitive (W ("ip_update_dns"), ip != IP_MANUAL);
 	gtk_widget_set_sensitive (W ("ip_table"), ip == IP_MANUAL);
 }
 
@@ -964,7 +990,7 @@ ip_config_menu_cb (GtkWidget *w, gpointer data)
 
 	connection_set_modified (cxn, TRUE);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (W ("status_dhcp_dns")),
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (W ("ip_update_dns")),
 				      ip != IP_MANUAL);
 
 	update_ip_config (cxn);
@@ -1000,6 +1026,7 @@ fill_ip (XstConnection *cxn)
 
 	update_ip_config (cxn);
 
+	SET_BOOL ("ip_", update_dns);
 	SET_STR ("ip_", address);
 	SET_STR ("ip_", netmask);
 	SET_STR ("ip_", gateway);
@@ -1027,8 +1054,8 @@ fill_ppp_adv (XstConnection *cxn)
 	SET_STR ("ppp_", serial_port);
 	SET_BOOL ("ppp_", stupid);
 	SET_BOOL ("ppp_", set_default_gw);
-	SET_BOOL_NOT ("ppp_", peerdns);
-	gtk_signal_emit_by_name (GTK_OBJECT (W ("ppp_peerdns")), "toggled");
+	SET_BOOL_NOT ("ppp_", update_dns);
+	gtk_signal_emit_by_name (GTK_OBJECT (W ("ppp_update_dns")), "toggled");
 	SET_STR ("ppp_", dns1);
 	SET_STR ("ppp_", dns2);
 	SET_STR ("ppp_", ppp_options);
@@ -1062,7 +1089,7 @@ hookup_callbacks (XstConnection *cxn)
 		{ "on_connection_modified", on_connection_modified },
 		{ "on_connection_config_dialog_destroy", on_connection_config_dialog_destroy },
 		{ "on_wvlan_adhoc_toggled", on_wvlan_adhoc_toggled },
-		{ "on_ppp_peerdns_toggled", on_ppp_peerdns_toggled },
+		{ "on_ppp_update_dns_toggled", on_ppp_update_dns_toggled },
 		{ NULL }
 	};
 	struct { char *name; EditableFilterRules rule; }
@@ -1089,7 +1116,7 @@ connection_configure (XstConnection *cxn)
 {
 	GtkWidget *nb;
 /*	GtkWidget *hb, *qm;*/
-	char *s;
+	gchar *s, *smart_dhcpcd;
 
 	if (cxn->window) {
 		gtk_widget_show (cxn->window);
@@ -1112,6 +1139,14 @@ connection_configure (XstConnection *cxn)
 	fill_general (cxn);
 	fill_ip      (cxn);
 
+	smart_dhcpcd = xst_xml_get_child_content (cxn->node->parent, "smartdhcpcd");
+	if (!smart_dhcpcd || !atoi (smart_dhcpcd)) {
+		GtkWidget *w = W("ip_update_dns");
+		gtk_widget_hide (w);
+	}
+
+	g_free (smart_dhcpcd);
+
 	/* would like to do this as a switch */
 	nb = W ("connection_nb");
 	if (cxn->type == XST_CONNECTION_PPP) {
@@ -1171,6 +1206,7 @@ connection_save_to_node (XstConnection *cxn, xmlNode *root)
 	connection_xml_save_boolean_to_node (node, "user", cxn->user);
 	connection_xml_save_boolean_to_node (node, "auto", cxn->autoboot);
 	connection_xml_save_boolean_to_node (node, "enabled", cxn->enabled);
+	connection_xml_save_boolean_to_node (node, "update_dns", cxn->update_dns);
 
 	/* TCP/IP general paramaters */
 	connection_xml_save_str_to_node (node, "address", cxn->address);
@@ -1198,7 +1234,6 @@ connection_save_to_node (XstConnection *cxn, xmlNode *root)
 		connection_xml_save_boolean_to_node (node, "persist", cxn->persist);
 		connection_xml_save_str_to_node (node, "serial_port", cxn->serial_port);
 		connection_xml_save_boolean_to_node (node, "set_default_gw", cxn->set_default_gw);
-		connection_xml_save_boolean_to_node (node, "peerdns", cxn->peerdns);
 		connection_xml_save_str_to_node (node, "dns1", cxn->dns1);
 		connection_xml_save_str_to_node (node, "dns2", cxn->dns2);
 		connection_xml_save_str_to_node (node, "ppp_options", cxn->ppp_options);
