@@ -23,7 +23,7 @@
 #endif
 
 #include <gnome.h>
-#include "global.h"
+#include "xst.h"
 #include <gal/e-table/e-table-scrolled.h>
 #include <gal/e-table/e-table-memory.h>
 #include <gal/e-table/e-table-memory-callbacks.h>
@@ -44,6 +44,7 @@
 typedef gboolean  (XstValidatorFunc)(gpointer val);
 typedef gchar    *(XstFilterFunc)   (gchar *val);
 typedef void      (XstChangedFunc)  (gpointer val);
+typedef gpointer  (XstValueFunc)    (xmlNodePtr node);
 
 typedef enum
 {
@@ -55,17 +56,35 @@ XstTableColType;
 
 typedef struct
 {
-	gchar *xml_tag;
-	XstTableColType type;
-	XstFilterFunc *filter;
+	gchar            *xml_tag;
+
+	XstTableColType   type;
+	XstFilterFunc    *filter;
 	XstValidatorFunc *validator;
-	XstChangedFunc *changed;
+	XstChangedFunc   *changed;
+	XstValueFunc     *value;
 }
 XstTableColDef;
 
-static void set_mount_button_label_neg    (gboolean unmount);
-static void mount_button_from_import_node (xmlNodePtr node);
-static void export_set_writable           (gboolean writable);
+static void     set_mount_button_label_neg    (gboolean unmount);
+static void     mount_button_from_import_node (xmlNodePtr node);
+static void     export_set_writable           (gboolean writable);
+static gpointer export_get_writable           (xmlNodePtr node);
+
+#ifdef JUST_FOR_TRANSLATORS
+static char *list [] = {
+	N_("Host"),
+	N_("Path"),
+	N_("Mount Point"),
+	N_("Mounted"),
+	N_("Name"),
+	N_("Path"),
+	N_("Clients"),
+	N_("Writable"),
+	N_("Pattern"),
+	N_("Write"),
+};
+#endif
 
 extern XstTool *tool;
 
@@ -156,19 +175,19 @@ static const gchar *export_clients_basic_state =
 
 XstTableColDef imports_table_col_def [] =
 {
-	{ "host",    XST_TABLE_COL_STRING, xst_util_nice_hostname_dup, validate_import_host_name,   NULL },
-	{ "path",    XST_TABLE_COL_STRING, xst_util_nice_path_dup,     validate_import_remote_path, NULL },
-	{ "point",   XST_TABLE_COL_STRING, xst_util_nice_path_dup,     validate_import_local_path,  NULL },
-	{ "mounted", XST_TABLE_COL_STATE,  NULL,                       NULL,                        set_mount_button_label_neg },
+	{ "host",    XST_TABLE_COL_STRING, xst_util_nice_hostname_dup, validate_import_host_name,   NULL, NULL },
+	{ "path",    XST_TABLE_COL_STRING, xst_util_nice_path_dup,     validate_import_remote_path, NULL, NULL },
+	{ "point",   XST_TABLE_COL_STRING, xst_util_nice_path_dup,     validate_import_local_path,  NULL, NULL },
+	{ "mounted", XST_TABLE_COL_STATE,  NULL,                       NULL,                        set_mount_button_label_neg, NULL },
 	{ NULL }
 };
 
 XstTableColDef exports_table_col_def [] =
 {
-	{ "name",     XST_TABLE_COL_STRING,       NULL,                   validate_export_name,       NULL },
-	{ "path",     XST_TABLE_COL_STRING,       xst_util_nice_path_dup, validate_export_local_path, NULL },
-/*	{ "clients",  XST_TABLE_COL_STRING_BLANK, NULL,                   NULL,                       NULL }, */
-	{ "write", XST_TABLE_COL_STATE,        NULL,                   NULL,                       export_set_writable },
+	{ "name",     XST_TABLE_COL_STRING,       NULL,                   validate_export_name,       NULL, NULL },
+	{ "path",     XST_TABLE_COL_STRING,       xst_util_nice_path_dup, validate_export_local_path, NULL, NULL },
+/*	{ "clients",  XST_TABLE_COL_STRING_BLANK, NULL,                   NULL,                       NULL, NULL }, */
+	{ "write",    XST_TABLE_COL_STATE,        NULL,                   NULL,                       export_set_writable, export_get_writable },
 	{ NULL }
 };
 
@@ -211,8 +230,8 @@ XstWidgetUserPolicy export_clients_selected_policies[] =
 
 XstWidgetUserPolicy export_clients_unselected_policies[] =
 {
-	{ "exports_clients_update_button", XST_WIDGET_MODE_INSENSITIVE },
-	{ "exports_clients_delete_button", XST_WIDGET_MODE_INSENSITIVE },
+	{ "export_clients_update_button", XST_WIDGET_MODE_INSENSITIVE },
+	{ "export_clients_delete_button", XST_WIDGET_MODE_INSENSITIVE },
 	{ NULL }
 };
 
@@ -269,13 +288,13 @@ set_value_at_with_col_def (ETableModel *etm, int col, int row, const void *val,
 	xst_dialog_modify (tool->main_dialog);
 }
 
-static void *
+static gpointer
 value_at_with_col_def (ETableModel *etm, int col, int row, void *data)
 {
 	XstTableColDef *def_table;
 	XstTableColDef *def;
 	xmlNodePtr node;
-	void *ret;
+	gpointer ret;
 
 	def_table = data;
 	def = &def_table [col];
@@ -288,7 +307,11 @@ value_at_with_col_def (ETableModel *etm, int col, int row, void *data)
 
 	node = e_table_memory_get_data (E_TABLE_MEMORY (etm), row);
 
-	if (def->type == XST_TABLE_COL_STRING)
+	if (def->value)
+	{
+		ret = def->value (node);
+	}
+	else if (def->type == XST_TABLE_COL_STRING)
 	{
 		if (!node)
 			return g_strdup ("");
@@ -751,7 +774,7 @@ set_mount_button_label (gboolean mount)
 	button = xst_dialog_get_widget (tool->main_dialog, "imports_mount_button");
 	g_return_if_fail (button != NULL);
 
-	gtk_widget_set (button, "label", mount ? "Mount" : "Unmount", NULL);
+	gtk_widget_set (button, "label", mount ? _("Mount") : _("Unmount"), NULL);
 }
 
 static void
@@ -865,35 +888,19 @@ import_table_mount_selected_node_toggle_cb (void)
 
 /* --- */
 
-
-static void
-export_set_general_write (xmlNodePtr export_node, gboolean general_write)
-{
-	xmlNodePtr node;
-
-	for (node = xst_xml_element_find_first (export_node, "allow"); node;
-	     node = xst_xml_element_find_next (node, "allow"))
-	{
-		gchar *pattern;
-
-		pattern = xst_xml_get_child_content (node, "pattern");
-		if (pattern && !strcmp (pattern, "0.0.0.0/0"))
-		{
-			xst_xml_element_set_state (node, "write", general_write);
-			break;
-		}
-		else if (pattern)
-			g_free (pattern);
-	}
-}
-
 static void
 export_set_writable (gboolean writable)
 {
 	xmlNodePtr node;
 
 	node = export_table_get_selected_node ();
-	export_set_general_write (node, writable);
+	shares_xml_export_set_general_write (node, writable);
+}
+
+static gpointer
+export_get_writable (xmlNodePtr node)
+{
+	return shares_xml_export_get_general_write (node);
 }
 
 void
